package main

import (
	"fmt"
	"log"
	"math"
	"time"

	tea "github.com/charmbracelet/bubbletea"
	"github.com/charmbracelet/lipgloss"
	"go.bug.st/serial/enumerator"
)

type model struct {
	values []float64 // Data points to plot
	width  int       // Width of the plot
	height int       // Height of the plot
	time   float64   // Time variable for animation
}
type TickMsg time.Time

func doTick() tea.Cmd {
	return tea.Tick(time.Second/30, func(t time.Time) tea.Msg {
		return TickMsg(t)
	})
}

// Initialize the model with some data
func initialModel() model {
	return model{
		values: make([]float64, 100), // More points for higher resolution
		width:  70,
		height: 25,
		time:   0.0,
	}
}

func (m model) Init() tea.Cmd {
	return doTick()
}

// Update function (required by bubbletea)
func (m model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
	switch msg := msg.(type) {
	case tea.KeyMsg:
		switch msg.String() {
		case "q", "ctrl+c":
			return m, tea.Quit
		}
	case TickMsg:

		// Update the time variable to animate the sine wave
		m.time += 0.1
		// Generate new sine wave values
		for i := range m.values {
			m.values[i] = math.Sin(float64(i)/10.0 + m.time)
		}
		return m, doTick()
	}
	return m, nil
}

// Interpolate between two colors based on a factor (0 to 1)
func interpolateColor(color1, color2 lipgloss.Color, factor float64) lipgloss.Color {
	r1, g1, b1 := hexToRGB(string(color1))
	r2, g2, b2 := hexToRGB(string(color2))
	r := uint8(float64(r1) + (float64(r2)-float64(r1))*factor)
	g := uint8(float64(g1) + (float64(g2)-float64(g1))*factor)
	b := uint8(float64(b1) + (float64(b2)-float64(b1))*factor)
	return lipgloss.Color(fmt.Sprintf("#%02X%02X%02X", r, g, b))
}

// Convert hex color to RGB values
func hexToRGB(hex string) (uint8, uint8, uint8) {
	var r, g, b uint8
	fmt.Sscanf(hex, "#%02X%02X%02X", &r, &g, &b)
	return r, g, b
}

// View function (required by bubbletea)
func (m model) View() string {
	// Create a 2D grid to represent the plot
	grid := make([][]string, m.height)
	for i := range grid {
		grid[i] = make([]string, m.width)
		for j := range grid[i] {
			grid[i][j] = " " // Initialize with spaces
		}
	}

	// Normalize values to fit within the plot height
	maxValue := 1.0 // Sine wave ranges from -1 to 1
	minValue := -1.0
	scale := float64(m.height-1) / (maxValue - minValue)

	// Define the gradient colors
	colorGreen := lipgloss.Color("#00FF00")  // Green
	colorOrange := lipgloss.Color("#FF7F00") // Orange
	colorRed := lipgloss.Color("#FF0000")    // Red

	// Plot the values with higher resolution and color
	for x, value := range m.values {
		y := int((value - minValue) * scale)
		if y >= 0 && y < m.height && x < m.width {
			// Calculate the interpolation factor based on the y position
			factor := float64(y) / float64(m.height-1)
			var color lipgloss.Color
			if factor < 0.5 {
				// Interpolate between Green and Orange
				color = interpolateColor(colorGreen, colorOrange, factor*2)
			} else {
				// Interpolate between Orange and Red
				color = interpolateColor(colorOrange, colorRed, (factor-0.5)*2)
			}
			style := lipgloss.NewStyle().Foreground(color)
			// Use a thinner character for the line
			grid[m.height-1-y][x] = style.Render("│") // Vertical bar for a thin line
		}
	}

	// Draw a horizontal line at y = 0.0
	yZero := int((0.0 - minValue) * scale) // Calculate the y position for 0.0
	if yZero >= 0 && yZero < m.height {
		for x := 0; x < m.width; x++ {
			// Use a horizontal line character
			grid[m.height-1-yZero][x] = lipgloss.NewStyle().Foreground(lipgloss.Color("#FFFFFF")).Render("─")
		}
	}

	// Convert the grid to a string with axis labels
	var plot string

	// Add y-axis labels
	for i := 0; i < m.height; i++ {
		// Calculate the corresponding value for the y-axis
		value := minValue + (maxValue-minValue)*float64(m.height-1-i)/float64(m.height-1)
		// Format the value to 2 decimal places
		label := fmt.Sprintf("%.2f", value)
		// Add padding to align the labels
		label = fmt.Sprintf("%6s", label)
		plot += label + " "
		// Add the grid row
		for j := 0; j < m.width; j++ {
			plot += grid[i][j]
		}
		plot += "\n"
	}

	// Add x-axis labels (time)
	plot += "       " // Align with y-axis labels
	for i := 0; i < m.width; i += 10 {
		// Calculate the time value for the x-axis
		timeValue := m.time + float64(i)/10.0
		// Format the time value to 1 decimal place
		label := fmt.Sprintf("%.1f", timeValue)
		label = fmt.Sprintf("%-10s", label)
		plot += label
	}
	plot += "\n"

	// Style the plot with a border
	style := lipgloss.NewStyle().
		BorderStyle(lipgloss.RoundedBorder()).
		BorderForeground(lipgloss.Color("#FFFFFF")). // White border
		Padding(1, 2)
	return style.Render(plot)
}

func main() {

	// Initialize the model
	model := initialModel()

	// Start the bubbletea program
	p := tea.NewProgram(model, tea.WithAltScreen())
	// _, err = p.Run()
	// if err != nil {
	// 	fmt.Printf("Error running program: %v\n", err)
	// 	os.Exit(1)
	// }
	p.Kill()
	ports, err := enumerator.GetDetailedPortsList()
	if err != nil {
		log.Fatal(err)
	}
	if len(ports) == 0 {
		fmt.Println("No serial ports found!")
		return
	}
	for _, port := range ports {
		fmt.Printf("Found port: %s\n", port.Name)
		if port.IsUSB {
			fmt.Printf("   USB ID     %s:%s\n", port.VID, port.PID)
			fmt.Printf("   USB serial %s\n", port.SerialNumber)
		}
	}
}
